@@ -1,30 +1,22 @@
 # -*- Mode: Python; py-indent-offset: 4 -*-
-# coding=utf-8
 # vim: tabstop=4 shiftwidth=4 expandtab
 
-import sys
+import pygtk
+pygtk.require("2.0")
 
 import unittest
-import tempfile
-import shutil
-import os
-import locale
-import subprocess
 from gi.repository import GObject
 
+from datetime import datetime
+
+import sys
+sys.path.insert(0, "../")
+
 import gobject
 from gi.repository import GIMarshallingTests
 
-from compathelper import _bytes
-
-if sys.version_info < (3, 0):
-    CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
-    PY2_UNICODE_UTF8 = unicode(CONSTANT_UTF8, 'UTF-8')
-    CHAR_255='\xff'
-else:
-    CONSTANT_UTF8 = "const ♥ utf8"
-    CHAR_255=bytes([255])
 
+CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
 CONSTANT_NUMBER = 42
 
 
@@ -128,9 +120,9 @@ class TestUInt8(unittest.TestCase):
         number = Number(self.MAX)
 
         GIMarshallingTests.uint8_in(number)
-        GIMarshallingTests.uint8_in(CHAR_255)
 
         number.value += 1
+
         self.assertRaises(ValueError, GIMarshallingTests.uint8_in, number)
         self.assertRaises(ValueError, GIMarshallingTests.uint8_in, Number(-1))
 
@@ -410,7 +402,6 @@ class TestInt(unittest.TestCase):
     def test_int_inout(self):
         self.assertEquals(self.MIN, GIMarshallingTests.int_inout_max_min(Number(self.MAX)))
         self.assertEquals(self.MAX, GIMarshallingTests.int_inout_min_max(Number(self.MIN)))
-        self.assertRaises(TypeError, GIMarshallingTests.int_inout_min_max, Number(self.MIN), CONSTANT_NUMBER)
 
 
 class TestUInt(unittest.TestCase):
@@ -597,6 +588,25 @@ class TestDouble(unittest.TestCase):
         self.assertAlmostEquals(self.MIN, GIMarshallingTests.double_inout(Number(self.MAX)))
 
 
+class TestTimeT(unittest.TestCase):
+
+    DATETIME = datetime.fromtimestamp(1234567890)
+
+    def test_time_t_return(self):
+        self.assertEquals(self.DATETIME, GIMarshallingTests.time_t_return())
+
+    def test_time_t_in(self):
+        GIMarshallingTests.time_t_in(self.DATETIME)
+
+        self.assertRaises(TypeError, GIMarshallingTests.time_t_in, "self.DATETIME")
+
+    def test_time_t_out(self):
+        self.assertEquals(self.DATETIME, GIMarshallingTests.time_t_out())
+
+    def test_time_t_inout(self):
+        self.assertEquals(datetime.fromtimestamp(0), GIMarshallingTests.time_t_inout(self.DATETIME))
+
+
 class TestGType(unittest.TestCase):
 
     def test_gtype_return(self):
@@ -624,12 +634,13 @@ class TestUtf8(unittest.TestCase):
 
     def test_utf8_none_in(self):
         GIMarshallingTests.utf8_none_in(CONSTANT_UTF8)
-        if sys.version_info < (3, 0):
-            GIMarshallingTests.utf8_none_in(PY2_UNICODE_UTF8)
 
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, CONSTANT_NUMBER)
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, None)
 
+    def test_utf8_full_in(self):
+        GIMarshallingTests.utf8_full_in(CONSTANT_UTF8)
+
     def test_utf8_none_out(self):
         self.assertEquals(CONSTANT_UTF8, GIMarshallingTests.utf8_none_out())
 
@@ -678,10 +689,6 @@ class TestArray(unittest.TestCase):
     def test_array_in(self):
         GIMarshallingTests.array_in(Sequence([-1, 0, 1, 2]))
 
-    def test_array_uint8_in(self):
-        GIMarshallingTests.array_uint8_in(Sequence([97, 98, 99, 100]))
-        GIMarshallingTests.array_uint8_in(_bytes("abcd"))
-
     def test_array_out(self):
         self.assertEquals([-1, 0, 1, 2], GIMarshallingTests.array_out())
 
@@ -768,6 +775,12 @@ class TestGArray(unittest.TestCase):
     def test_garray_utf8_none_in(self):
         GIMarshallingTests.garray_utf8_none_in(Sequence(['0', '1', '2']))
 
+    def test_garray_utf8_container_in(self):
+        GIMarshallingTests.garray_utf8_container_in(Sequence(['0', '1', '2']))
+
+    def test_garray_utf8_full_in(self):
+        GIMarshallingTests.garray_utf8_full_in(Sequence(['0', '1', '2']))
+
     def test_garray_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.garray_utf8_none_out())
 
@@ -812,6 +825,12 @@ class TestGList(unittest.TestCase):
     def test_glist_utf8_none_in(self):
         GIMarshallingTests.glist_utf8_none_in(Sequence(('0', '1', '2')))
 
+    def test_glist_utf8_container_in(self):
+        GIMarshallingTests.glist_utf8_container_in(Sequence(('0', '1', '2')))
+
+    def test_glist_utf8_full_in(self):
+        GIMarshallingTests.glist_utf8_full_in(Sequence(('0', '1', '2')))
+
     def test_glist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.glist_utf8_none_out())
 
@@ -856,6 +875,12 @@ class TestGSList(unittest.TestCase):
     def test_gslist_utf8_none_in(self):
         GIMarshallingTests.gslist_utf8_none_in(Sequence(('0', '1', '2')))
 
+    def test_gslist_utf8_container_in(self):
+        GIMarshallingTests.gslist_utf8_container_in(Sequence(('0', '1', '2')))
+
+    def test_gslist_utf8_full_in(self):
+        GIMarshallingTests.gslist_utf8_full_in(Sequence(('0', '1', '2')))
+
     def test_gslist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.gslist_utf8_none_out())
 
@@ -901,6 +926,12 @@ class TestGHashTable(unittest.TestCase):
     def test_ghashtable_utf8_none_in(self):
         GIMarshallingTests.ghashtable_utf8_none_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
 
+    def test_ghashtable_utf8_container_in(self):
+        GIMarshallingTests.ghashtable_utf8_container_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
+
+    def test_ghashtable_utf8_full_in(self):
+        GIMarshallingTests.ghashtable_utf8_full_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
+
     def test_ghashtable_utf8_none_out(self):
         self.assertEquals({'-1': '1', '0': '0', '1': '-1', '2': '-2'}, GIMarshallingTests.ghashtable_utf8_none_out())
 
@@ -930,30 +961,20 @@ class TestGValue(unittest.TestCase):
 
     def test_gvalue_in(self):
         GIMarshallingTests.gvalue_in(42)
-        value = GObject.Value()
-        value.init(GObject.TYPE_INT)
-        value.set_int(42)
-        GIMarshallingTests.gvalue_in(value)
+        self.assertRaises(TypeError, GIMarshallingTests.gvalue_in, None)
 
     def test_gvalue_out(self):
         self.assertEquals(42, GIMarshallingTests.gvalue_out())
 
     def test_gvalue_inout(self):
         self.assertEquals('42', GIMarshallingTests.gvalue_inout(42))
-        value = GObject.Value()
-        value.init(GObject.TYPE_INT)
-        value.set_int(42)
-        self.assertEquals('42', GIMarshallingTests.gvalue_inout(value))
+
 
 class TestGClosure(unittest.TestCase):
 
     def test_gclosure_in(self):
         GIMarshallingTests.gclosure_in(lambda: 42)
 
-        # test passing a closure between two C calls
-        closure = GIMarshallingTests.gclosure_return()
-        GIMarshallingTests.gclosure_in(closure)
-
         self.assertRaises(TypeError, GIMarshallingTests.gclosure_in, 42)
         self.assertRaises(TypeError, GIMarshallingTests.gclosure_in, None)
 
@@ -965,31 +986,6 @@ class TestPointer(unittest.TestCase):
 
 class TestEnum(unittest.TestCase):
 
-    @classmethod
-    def setUpClass(cls):
-        '''Run tests under a test locale.
-
-        Upper case conversion of member names should not be locale specific;
-        e.  g. in Turkish, "i".upper() == "i", which gives results like "iNVALiD"
-
-        Run test under a locale which defines toupper('a') == 'a'
-        '''
-        cls.locale_dir = tempfile.mkdtemp()
-        subprocess.check_call(['localedef', '-i',
-            os.path.join(os.path.dirname(os.path.realpath(__file__)), 'te_ST@nouppera'),
-            '-c', '-f', 'UTF-8', os.path.join(cls.locale_dir, 'te_ST.UTF-8@nouppera')])
-        os.environ['LOCPATH'] = cls.locale_dir
-        locale.setlocale(locale.LC_ALL, 'te_ST.UTF-8@nouppera')
-
-    @classmethod
-    def tearDownClass(cls):
-        locale.setlocale(locale.LC_ALL, 'C')
-        shutil.rmtree(cls.locale_dir)
-        try:
-            del os.environ['LOCPATH']
-        except KeyError:
-            pass
-
     def test_enum(self):
         self.assertTrue(issubclass(GIMarshallingTests.Enum, int))
         self.assertTrue(isinstance(GIMarshallingTests.Enum.VALUE1, GIMarshallingTests.Enum))
@@ -997,20 +993,10 @@ class TestEnum(unittest.TestCase):
         self.assertTrue(isinstance(GIMarshallingTests.Enum.VALUE3, GIMarshallingTests.Enum))
         self.assertEquals(42, GIMarshallingTests.Enum.VALUE3)
 
-    def test_value_nick_and_name(self):
-        self.assertEqual(GIMarshallingTests.Enum.VALUE1.value_nick, 'value1')
-        self.assertEqual(GIMarshallingTests.Enum.VALUE2.value_nick, 'value2')
-        self.assertEqual(GIMarshallingTests.Enum.VALUE3.value_nick, 'value3')
-
-        self.assertEqual(GIMarshallingTests.Enum.VALUE1.value_name, 'GI_MARSHALLING_TESTS_ENUM_VALUE1')
-        self.assertEqual(GIMarshallingTests.Enum.VALUE2.value_name, 'GI_MARSHALLING_TESTS_ENUM_VALUE2')
-        self.assertEqual(GIMarshallingTests.Enum.VALUE3.value_name, 'GI_MARSHALLING_TESTS_ENUM_VALUE3')
-
     def test_enum_in(self):
         GIMarshallingTests.enum_in(GIMarshallingTests.Enum.VALUE3)
-        GIMarshallingTests.enum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 43)
+        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 42)
         self.assertRaises(TypeError, GIMarshallingTests.enum_in, 'GIMarshallingTests.Enum.VALUE3')
 
     def test_enum_out(self):
@@ -1023,16 +1009,6 @@ class TestEnum(unittest.TestCase):
         self.assertTrue(isinstance(enum, GIMarshallingTests.Enum))
         self.assertEquals(enum, GIMarshallingTests.Enum.VALUE1)
 
-    def test_enum_second(self):
-        # check for the bug where different non-gtype enums share the same class
-        self.assertNotEqual(GIMarshallingTests.Enum, GIMarshallingTests.SecondEnum)
-
-        # check that values are not being shared between different enums
-        self.assertTrue(hasattr(GIMarshallingTests.SecondEnum, "SECONDVALUE1"))
-        self.assertRaises(AttributeError, getattr, GIMarshallingTests.Enum, "SECONDVALUE1")
-        self.assertTrue(hasattr(GIMarshallingTests.Enum, "VALUE1"))
-        self.assertRaises(AttributeError, getattr, GIMarshallingTests.SecondEnum, "VALUE1")
-
 
 class TestGEnum(unittest.TestCase):
 
@@ -1043,20 +1019,10 @@ class TestGEnum(unittest.TestCase):
         self.assertTrue(isinstance(GIMarshallingTests.GEnum.VALUE3, GIMarshallingTests.GEnum))
         self.assertEquals(42, GIMarshallingTests.GEnum.VALUE3)
 
-    def test_value_nick_and_name(self):
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE1.value_nick, 'value1')
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE2.value_nick, 'value2')
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE3.value_nick, 'value3')
-
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE1.value_name, 'GI_MARSHALLING_TESTS_GENUM_VALUE1')
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE2.value_name, 'GI_MARSHALLING_TESTS_GENUM_VALUE2')
-        self.assertEqual(GIMarshallingTests.GEnum.VALUE3.value_name, 'GI_MARSHALLING_TESTS_GENUM_VALUE3')
-
     def test_genum_in(self):
         GIMarshallingTests.genum_in(GIMarshallingTests.GEnum.VALUE3)
-        GIMarshallingTests.genum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 43)
+        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 42)
         self.assertRaises(TypeError, GIMarshallingTests.genum_in, 'GIMarshallingTests.GEnum.VALUE3')
 
     def test_genum_out(self):
@@ -1077,24 +1043,10 @@ class TestGFlags(unittest.TestCase):
         self.assertTrue(isinstance(GIMarshallingTests.Flags.VALUE1, GIMarshallingTests.Flags))
         self.assertTrue(isinstance(GIMarshallingTests.Flags.VALUE2, GIMarshallingTests.Flags))
         self.assertTrue(isinstance(GIMarshallingTests.Flags.VALUE3, GIMarshallingTests.Flags))
-        # __or__() operation should still return an instance, not an int.
-        self.assertTrue(isinstance(GIMarshallingTests.Flags.VALUE1 | GIMarshallingTests.Flags.VALUE2,
-                                   GIMarshallingTests.Flags))
         self.assertEquals(1 << 1, GIMarshallingTests.Flags.VALUE2)
 
-    def test_value_nick_and_name(self):
-        self.assertEqual(GIMarshallingTests.Flags.VALUE1.first_value_nick, 'value1')
-        self.assertEqual(GIMarshallingTests.Flags.VALUE2.first_value_nick, 'value2')
-        self.assertEqual(GIMarshallingTests.Flags.VALUE3.first_value_nick, 'value3')
-
-        self.assertEqual(GIMarshallingTests.Flags.VALUE1.first_value_name, 'GI_MARSHALLING_TESTS_FLAGS_VALUE1')
-        self.assertEqual(GIMarshallingTests.Flags.VALUE2.first_value_name, 'GI_MARSHALLING_TESTS_FLAGS_VALUE2')
-        self.assertEqual(GIMarshallingTests.Flags.VALUE3.first_value_name, 'GI_MARSHALLING_TESTS_FLAGS_VALUE3')
-
     def test_flags_in(self):
         GIMarshallingTests.flags_in(GIMarshallingTests.Flags.VALUE2)
-        # result of __or__() operation should still be valid instance, not an int.
-        GIMarshallingTests.flags_in(GIMarshallingTests.Flags.VALUE2 | GIMarshallingTests.Flags.VALUE2)
         GIMarshallingTests.flags_in_zero(Number(0))
 
         self.assertRaises(TypeError, GIMarshallingTests.flags_in, 1 << 1)
@@ -1110,45 +1062,6 @@ class TestGFlags(unittest.TestCase):
         self.assertTrue(isinstance(flags, GIMarshallingTests.Flags))
         self.assertEquals(flags, GIMarshallingTests.Flags.VALUE1)
 
-class TestNoTypeFlags(unittest.TestCase):
-
-    def test_flags(self):
-        self.assertTrue(issubclass(GIMarshallingTests.NoTypeFlags, GObject.GFlags))
-        self.assertTrue(isinstance(GIMarshallingTests.NoTypeFlags.VALUE1, GIMarshallingTests.NoTypeFlags))
-        self.assertTrue(isinstance(GIMarshallingTests.NoTypeFlags.VALUE2, GIMarshallingTests.NoTypeFlags))
-        self.assertTrue(isinstance(GIMarshallingTests.NoTypeFlags.VALUE3, GIMarshallingTests.NoTypeFlags))
-        # __or__() operation should still return an instance, not an int.
-        self.assertTrue(isinstance(GIMarshallingTests.NoTypeFlags.VALUE1 | GIMarshallingTests.NoTypeFlags.VALUE2,
-                                   GIMarshallingTests.NoTypeFlags))
-        self.assertEquals(1 << 1, GIMarshallingTests.NoTypeFlags.VALUE2)
-
-    def test_value_nick_and_name(self):
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE1.first_value_nick, 'value1')
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE2.first_value_nick, 'value2')
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE3.first_value_nick, 'value3')
-
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE1.first_value_name, 'GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE1')
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE2.first_value_name, 'GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE2')
-        self.assertEqual(GIMarshallingTests.NoTypeFlags.VALUE3.first_value_name, 'GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE3')
-
-    def test_flags_in(self):
-        GIMarshallingTests.no_type_flags_in(GIMarshallingTests.NoTypeFlags.VALUE2)
-        GIMarshallingTests.no_type_flags_in(GIMarshallingTests.NoTypeFlags.VALUE2 | GIMarshallingTests.NoTypeFlags.VALUE2)
-        GIMarshallingTests.no_type_flags_in_zero(Number(0))
-
-        self.assertRaises(TypeError, GIMarshallingTests.no_type_flags_in, 1 << 1)
-        self.assertRaises(TypeError, GIMarshallingTests.no_type_flags_in, 'GIMarshallingTests.NoTypeFlags.VALUE2')
-
-    def test_flags_out(self):
-        flags = GIMarshallingTests.no_type_flags_out()
-        self.assertTrue(isinstance(flags, GIMarshallingTests.NoTypeFlags))
-        self.assertEquals(flags, GIMarshallingTests.NoTypeFlags.VALUE2)
-
-    def test_flags_inout(self):
-        flags = GIMarshallingTests.no_type_flags_inout(GIMarshallingTests.NoTypeFlags.VALUE2)
-        self.assertTrue(isinstance(flags, GIMarshallingTests.NoTypeFlags))
-        self.assertEquals(flags, GIMarshallingTests.NoTypeFlags.VALUE1)
-
 
 class TestStructure(unittest.TestCase):
 
@@ -1184,7 +1097,7 @@ class TestStructure(unittest.TestCase):
         self.assertEquals(None, struct.pointer)
 
     def test_simple_struct_return(self):
-        struct = GIMarshallingTests.simple_struct_returnv()
+        struct = GIMarshallingTests.simple_struct_return()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
         self.assertEquals(6, struct.long_)
@@ -1197,17 +1110,40 @@ class TestStructure(unittest.TestCase):
         struct.long_ = 6
         struct.int8 = 7
 
-        GIMarshallingTests.SimpleStruct.inv(struct)
+        GIMarshallingTests.simple_struct_in(struct)
 
         del struct
 
         struct = GIMarshallingTests.NestedStruct()
 
-        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, struct)
+        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, struct)
 
         del struct
 
-        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, None)
+        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, None)
+
+    def test_simple_struct_out(self):
+        struct = GIMarshallingTests.simple_struct_out()
+
+        self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
+        self.assertEquals(6, struct.long_)
+        self.assertEquals(7, struct.int8)
+
+        del struct
+
+    def test_simple_struct_inout(self):
+        in_struct = GIMarshallingTests.SimpleStruct()
+        in_struct.long_ = 6
+        in_struct.int8 = 7
+
+        out_struct = GIMarshallingTests.simple_struct_inout(in_struct)
+
+        self.assertTrue(isinstance(out_struct, GIMarshallingTests.SimpleStruct))
+        self.assertEquals(7, out_struct.long_)
+        self.assertEquals(6, out_struct.int8)
+
+        del in_struct
+        del out_struct
 
     def test_simple_struct_method(self):
         struct = GIMarshallingTests.SimpleStruct()
@@ -1230,7 +1166,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_pointer_struct_return(self):
-        struct = GIMarshallingTests.pointer_struct_returnv()
+        struct = GIMarshallingTests.pointer_struct_return()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
         self.assertEquals(42, struct.long_)
@@ -1241,10 +1177,30 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.PointerStruct()
         struct.long_ = 42
 
-        struct.inv()
+        GIMarshallingTests.pointer_struct_in(struct)
 
         del struct
 
+    def test_pointer_struct_out(self):
+        struct = GIMarshallingTests.pointer_struct_out()
+
+        self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
+        self.assertEquals(42, struct.long_)
+
+        del struct
+
+    def test_pointer_struct_inout(self):
+        in_struct = GIMarshallingTests.PointerStruct()
+        in_struct.long_ = 42
+
+        out_struct = GIMarshallingTests.pointer_struct_inout(in_struct)
+
+        self.assertTrue(isinstance(out_struct, GIMarshallingTests.PointerStruct))
+        self.assertEquals(0, out_struct.long_)
+
+        del in_struct
+        del out_struct
+
     def test_boxed_struct(self):
         self.assertTrue(issubclass(GIMarshallingTests.BoxedStruct, GObject.GBoxed))
 
@@ -1272,7 +1228,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_boxed_struct_return(self):
-        struct = GIMarshallingTests.boxed_struct_returnv()
+        struct = GIMarshallingTests.boxed_struct_return()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.BoxedStruct))
         self.assertEquals(42, struct.long_)
@@ -1284,7 +1240,7 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.BoxedStruct()
         struct.long_ = 42
 
-        struct.inv()
+        GIMarshallingTests.boxed_struct_in(struct)
 
         del struct
 
@@ -1320,7 +1276,7 @@ class TestStructure(unittest.TestCase):
         del new_union
 
     def test_union_return(self):
-        union = GIMarshallingTests.union_returnv()
+        union = GIMarshallingTests.union_return()
 
         self.assertTrue(isinstance(union, GIMarshallingTests.Union))
         self.assertEquals(42, union.long_)
@@ -1331,10 +1287,30 @@ class TestStructure(unittest.TestCase):
         union = GIMarshallingTests.Union()
         union.long_ = 42
 
-        union.inv()
+        GIMarshallingTests.union_in(union)
 
         del union
 
+    def test_union_out(self):
+        union = GIMarshallingTests.union_out()
+
+        self.assertTrue(isinstance(union, GIMarshallingTests.Union))
+        self.assertEquals(42, union.long_)
+
+        del union
+
+    def test_union_inout(self):
+        in_union = GIMarshallingTests.Union()
+        in_union.long_ = 42
+
+        out_union = GIMarshallingTests.union_inout(in_union)
+
+        self.assertTrue(isinstance(out_union, GIMarshallingTests.Union))
+        self.assertEquals(0, out_union.long_)
+
+        del in_union
+        del out_union
+
     def test_union_method(self):
         union = GIMarshallingTests.Union()
         union.long_ = 42
@@ -1410,44 +1386,49 @@ class TestGObject(unittest.TestCase):
 #        self.assertEquals(object_.int_, 42)
 
     def test_object_none_return(self):
-        object_ = GIMarshallingTests.Object.none_return()
+        object_ = GIMarshallingTests.object_none_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
     def test_object_full_return(self):
-        object_ = GIMarshallingTests.Object.full_return()
+        object_ = GIMarshallingTests.object_full_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_in(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        GIMarshallingTests.Object.none_in(object_)
+        GIMarshallingTests.object_none_in(object_)
         self.assertEquals(object_.__grefcount__, 1)
 
         object_ = GIMarshallingTests.SubObject(int = 42)
-        GIMarshallingTests.Object.none_in(object_)
+        GIMarshallingTests.object_none_in(object_)
 
         object_ = GObject.GObject()
-        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, object_)
+        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, object_)
 
-        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, None)
+        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, None)
+
+    def test_object_full_in(self):
+        object_ = GIMarshallingTests.Object(int = 42)
+        GIMarshallingTests.object_full_in(object_)
+        self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_out(self):
-        object_ = GIMarshallingTests.Object.none_out()
+        object_ = GIMarshallingTests.object_none_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
-        new_object = GIMarshallingTests.Object.none_out()
+        new_object = GIMarshallingTests.object_none_out()
         self.assertTrue(new_object is object_)
 
     def test_object_full_out(self):
-        object_ = GIMarshallingTests.Object.full_out()
+        object_ = GIMarshallingTests.object_full_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.Object.none_inout(object_)
+        new_object = GIMarshallingTests.object_none_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
@@ -1456,20 +1437,20 @@ class TestGObject(unittest.TestCase):
         self.assertEquals(object_.__grefcount__, 1)
         self.assertEquals(new_object.__grefcount__, 2)
 
-        new_new_object = GIMarshallingTests.Object.none_inout(object_)
+        new_new_object = GIMarshallingTests.object_none_inout(object_)
         self.assertTrue(new_new_object is new_object)
 
-        GIMarshallingTests.Object.none_inout(GIMarshallingTests.SubObject(int = 42))
+        GIMarshallingTests.object_none_inout(GIMarshallingTests.SubObject(int = 42))
 
     def test_object_full_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.Object.full_inout(object_)
+        new_object = GIMarshallingTests.object_full_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
         self.assertFalse(object_ is new_object)
 
-        self.assertEquals(object_.__grefcount__, 2)
+        self.assertEquals(object_.__grefcount__, 1)
         self.assertEquals(new_object.__grefcount__, 1)
 
 # FIXME: Doesn't actually return the same object.
@@ -1483,6 +1464,8 @@ class TestGObject(unittest.TestCase):
 class TestPythonGObject(unittest.TestCase):
 
     class Object(GIMarshallingTests.Object):
+        __gtype_name__ = "Object"
+
         def __init__(self, int):
             GIMarshallingTests.Object.__init__(self)
             self.val = None
@@ -1498,16 +1481,7 @@ class TestPythonGObject(unittest.TestCase):
             return 42
 
         def do_method_with_default_implementation(self, int8):
-            GIMarshallingTests.Object.do_method_with_default_implementation(self, int8)
-            self.props.int += int8
-
-    class SubObject(GIMarshallingTests.SubObject):
-        def __init__(self, int):
-            GIMarshallingTests.SubObject.__init__(self)
-            self.val = None
-
-        def do_method_with_default_implementation(self, int8):
-            self.val = int8
+            self.props.int = int8 * 2
 
     def test_object(self):
         self.assertTrue(issubclass(self.Object, GIMarshallingTests.Object))
@@ -1525,9 +1499,11 @@ class TestPythonGObject(unittest.TestCase):
         self.assertEqual(object_.method_int8_out(), 42)
 
         object_.method_with_default_implementation(42)
-        self.assertEqual(object_.props.int, 84)
+        self.assertEqual(object_.val, 84)
 
         class ObjectWithoutVFunc(GIMarshallingTests.Object):
+            __gtype_name__ = 'ObjectWithoutVFunc'
+
             def __init__(self, int):
                 GIMarshallingTests.Object.__init__(self)
 
@@ -1535,11 +1511,6 @@ class TestPythonGObject(unittest.TestCase):
         object_.method_with_default_implementation(84)
         self.assertEqual(object_.props.int, 84)
 
-    def test_subobject_parent_vfunc(self):
-        object_ = self.SubObject(int = 81)
-        object_.method_with_default_implementation(87)
-        self.assertEquals(object_.val, 87)
-
     def test_dynamic_module(self):
         from gi.module import DynamicGObjectModule
         self.assertTrue(isinstance(GObject, DynamicGObjectModule))
@@ -1548,44 +1519,6 @@ class TestPythonGObject(unittest.TestCase):
         # compare a static gobject attr with a dynamic GObject attr
         self.assertEquals(GObject.GObject, gobject.GObject)
 
-    def test_subobject_non_vfunc_do_method(self):
-        class PythonObjectWithNonVFuncDoMethod:
-            def do_not_a_vfunc(self):
-                return 5
-
-        class ObjectOverrideNonVFuncDoMethod(GIMarshallingTests.Object, PythonObjectWithNonVFuncDoMethod):
-            def do_not_a_vfunc(self):
-                value = super(ObjectOverrideNonVFuncDoMethod, self).do_not_a_vfunc()
-                return 13 + value
-
-        object_ = ObjectOverrideNonVFuncDoMethod()
-        self.assertEquals(18, object_.do_not_a_vfunc())
-
-    def test_native_function_not_set_in_subclass_dict(self):
-        # Previously, GI was setting virtual functions on the class as well
-        # as any *native* class that subclasses it. Here we check that it is only
-        # set on the class that the method is originally from.
-        self.assertTrue('do_method_with_default_implementation' in GIMarshallingTests.Object.__dict__)
-        self.assertTrue('do_method_with_default_implementation' not in GIMarshallingTests.SubObject.__dict__)
-
-        # Here we check that accessing a vfunc from the subclass returns the same wrapper object,
-        # meaning that multiple wrapper objects have not been created for the same vfunc.
-        func1 = GIMarshallingTests.Object.do_method_with_default_implementation
-        func2 = GIMarshallingTests.SubObject.do_method_with_default_implementation
-        if sys.version_info < (3,0):
-            func1 = func1.im_func
-            func2 = func2.im_func
-            
-        self.assertTrue(func1 is func2)
-
-    def test_subobject_with_interface_and_non_vfunc_do_method(self):
-        # There was a bug for searching for vfuncs in interfaces. It was
-        # triggered by having a do_* method that wasn't overriding
-        # a native vfunc, as well as inheriting from an interface.
-        class GObjectSubclassWithInterface(GObject.GObject, GIMarshallingTests.Interface):
-            def do_method_not_a_vfunc(self):
-                pass
-
 class TestMultiOutputArgs(unittest.TestCase):
 
     def test_int_out_out(self):
@@ -1594,17 +1527,6 @@ class TestMultiOutputArgs(unittest.TestCase):
     def test_int_return_out(self):
         self.assertEquals((6, 7), GIMarshallingTests.int_return_out())
 
-class TestGErrorException(unittest.TestCase):
-    def test_gerror_exception(self):
-        self.assertRaises(GObject.GError, GIMarshallingTests.gerror)
-        try:
-            GIMarshallingTests.gerror()
-        except Exception:
-            etype, e = sys.exc_info()[:2]
-            self.assertEquals(e.domain, GIMarshallingTests.CONSTANT_GERROR_DOMAIN)
-            self.assertEquals(e.code, GIMarshallingTests.CONSTANT_GERROR_CODE)
-            self.assertEquals(e.message, GIMarshallingTests.CONSTANT_GERROR_MESSAGE)
-
 
 # Interface
 
@@ -1618,6 +1540,7 @@ class TestInterfaces(unittest.TestCase):
     def test_implementation(self):
 
         class TestInterfaceImpl(GObject.GObject, GIMarshallingTests.Interface):
+            __gtype_name__ = 'TestInterfaceImpl'
             def __init__(self):
                 GObject.GObject.__init__(self)
                 self.val = None
@@ -1634,42 +1557,24 @@ class TestInterfaces(unittest.TestCase):
         self.assertEquals(instance.val, 42)
 
         class TestInterfaceImplA(TestInterfaceImpl):
-            pass
+            __gtype_name__ = 'TestInterfaceImplA'
 
         class TestInterfaceImplB(TestInterfaceImplA):
-            pass
+            __gtype_name__ = 'TestInterfaceImplB'
 
         instance = TestInterfaceImplA()
         GIMarshallingTests.test_interface_test_int8_in(instance, 42)
         self.assertEquals(instance.val, 42)
 
-    def test_mro(self):
-        # there was a problem with Python bailing out because of
-        # http://en.wikipedia.org/wiki/Diamond_problem with interfaces,
-        # which shouldn't really be a problem.
+        def define_implementor_without_gtype():
+            class TestInterfaceImpl(gobject.GObject, GIMarshallingTests.Interface):
+                def __init__(self):
+                    gobject.GObject.__init__(self)
+                    self.val = None
 
-        class TestInterfaceImpl(GObject.GObject, GIMarshallingTests.Interface):
-            pass
-
-        class TestInterfaceImpl2(GIMarshallingTests.Interface,
-                                 TestInterfaceImpl):
-            pass
-
-        class TestInterfaceImpl3(TestInterfaceImpl,
-                                 GIMarshallingTests.Interface2):
-            pass
-
-
-class TestInterfaceClash(unittest.TestCase):
-
-    def test_clash(self):
-        def create_clash():
-            class TestClash(GObject.GObject, GIMarshallingTests.Interface, GIMarshallingTests.Interface2):
                 def do_test_int8_in(self, int8):
-                    pass
-            TestClash()
-
-        self.assertRaises(TypeError, create_clash)
+                    self.val = int8
+        self.assertRaises(RuntimeError, define_implementor_without_gtype)
 
 
 class TestOverrides(unittest.TestCase):
@@ -1689,7 +1594,7 @@ class TestOverrides(unittest.TestCase):
         del struct
 
         # Test that the overrides wrapper has been registered.
-        struct = GIMarshallingTests.overrides_struct_returnv()
+        struct = GIMarshallingTests.overrides_struct_return()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.OverridesStruct))
 
@@ -1710,43 +1615,10 @@ class TestOverrides(unittest.TestCase):
         self.assertEquals(6, object_.method())
 
         # Test that the overrides wrapper has been registered.
-        object_ = GIMarshallingTests.OverridesObject.returnv()
+        object_ = GIMarshallingTests.overrides_object_return()
 
         self.assertTrue(isinstance(object_, GIMarshallingTests.OverridesObject))
 
     def test_module_name(self):
         self.assertEquals(GIMarshallingTests.OverridesStruct.__module__, 'gi.overrides.GIMarshallingTests')
         self.assertEquals(GObject.InitiallyUnowned.__module__, 'gi.repository.GObject')
-
-class TestDir(unittest.TestCase):
-    def test_members_list(self):
-        list = dir(GIMarshallingTests)
-        self.assertTrue('OverridesStruct' in list)
-        self.assertTrue('BoxedStruct' in list)
-        self.assertTrue('OVERRIDES_CONSTANT' in list)
-        self.assertTrue('GEnum' in list)
-        self.assertTrue('int32_return_max' in list)
-
-    def test_modules_list(self):
-        import gi.repository
-        list = dir(gi.repository)
-        self.assertTrue('GIMarshallingTests' in list)
-
-        # FIXME: test to see if a module which was not imported is in the list
-        #        we should be listing every typelib we find, not just the ones
-        #        which are imported
-        #
-        #        to test this I recommend we compile a fake module which
-        #        our tests would never import and check to see if it is
-        #        in the list:
-        #
-        # self.assertTrue('DoNotImportDummyTests' in list)
-
-class TestGErrorArrayInCrash(unittest.TestCase):
-    # Previously there was a bug in invoke, in which C arrays were unwrapped
-    # from inside GArrays to be passed to the C function. But when a GError was
-    # set, invoke would attempt to free the C array as if it were a GArray.
-    # This crash is only for C arrays. It does not happen for C functions which
-    # take in GArrays. See https://bugzilla.gnome.org/show_bug.cgi?id=642708
-    def test_gerror_array_in_crash(self):
-        self.assertRaises(GObject.GError, GIMarshallingTests.gerror_array_in, [1, 2, 3])
